@@ -30,18 +30,19 @@
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <eel/eel-gtk-macros.h>
-#include <eel/eel-vfs-extensions.h>
-#include <libnautilus-private/nautilus-file-utilities.h>
-#include <libnautilus-private/nautilus-icon-names.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
 
-struct NautilusDesktopWindowDetails {
-	int dummy;
-};
+#include <eel/eel-vfs-extensions.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
+#include <libnautilus-private/nautilus-icon-names.h>
+#include <libnautilus-private/nautilus-global-preferences.h>
 
-static void set_wmspec_desktop_hint (GdkWindow *window);
+struct NautilusDesktopWindowDetails {
+	gulong size_changed_id;
+
+	gboolean loaded;
+};
 
 G_DEFINE_TYPE (NautilusDesktopWindow, nautilus_desktop_window, 
 	       NAUTILUS_TYPE_SPATIAL_WINDOW);
@@ -51,8 +52,9 @@ nautilus_desktop_window_init (NautilusDesktopWindow *window)
 {
 	GtkAction *action;
 	AtkObject *accessible;
-	
-	window->details = g_new0 (NautilusDesktopWindowDetails, 1);
+
+	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window, NAUTILUS_TYPE_DESKTOP_WINDOW,
+						       NautilusDesktopWindowDetails);
 
 	gtk_window_move (GTK_WINDOW (window), 0, 0);
 
@@ -75,8 +77,16 @@ nautilus_desktop_window_init (NautilusDesktopWindow *window)
 
 	/* Set the accessible name so that it doesn't inherit the cryptic desktop URI. */
 	accessible = gtk_widget_get_accessible (GTK_WIDGET (window));
-	if (accessible)
+
+	if (accessible) {
 		atk_object_set_name (accessible, _("Desktop"));
+	}
+
+	/* Monitor the preference to have the desktop */
+	/* point to the Unix home folder */
+	g_signal_connect_swapped (nautilus_preferences, "changed::" NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
+				  G_CALLBACK (nautilus_desktop_window_update_directory),
+				  window);
 }
 
 static gint
@@ -93,9 +103,10 @@ nautilus_desktop_window_update_directory (NautilusDesktopWindow *window)
 	
 	g_assert (NAUTILUS_IS_DESKTOP_WINDOW (window));
 	
-	NAUTILUS_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change = TRUE;
 	location = g_file_new_for_uri (EEL_DESKTOP_URI);
 	nautilus_window_go_to (NAUTILUS_WINDOW (window), location);
+	window->details->loaded = TRUE;
+
 	g_object_unref (location);
 }
 
@@ -145,18 +156,6 @@ nautilus_desktop_window_new (NautilusApplication *application,
 	return window;
 }
 
-static void
-finalize (GObject *object)
-{
-	NautilusDesktopWindow *window;
-
-	window = NAUTILUS_DESKTOP_WINDOW (object);
-
-	g_free (window->details);
-
-	G_OBJECT_CLASS (nautilus_desktop_window_parent_class)->finalize (object);
-}
-
 static void
 map (GtkWidget *widget)
 {
@@ -165,14 +164,15 @@ map (GtkWidget *widget)
 	gdk_window_lower (gtk_widget_get_window (widget));
 }
 
-
 static void
 unrealize (GtkWidget *widget)
 {
 	NautilusDesktopWindow *window;
+	NautilusDesktopWindowDetails *details;
 	GdkWindow *root_window;
 
 	window = NAUTILUS_DESKTOP_WINDOW (widget);
+	details = window->details;
 
 	root_window = gdk_screen_get_root_window (
 				gtk_window_get_screen (GTK_WINDOW (window)));
@@ -180,10 +180,12 @@ unrealize (GtkWidget *widget)
 	gdk_property_delete (root_window,
 			     gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", TRUE));
 
-	g_signal_handlers_disconnect_by_func (gtk_window_get_screen (GTK_WINDOW (window)),
-					      G_CALLBACK (nautilus_desktop_window_screen_size_changed),
-					      window);
-		
+	if (details->size_changed_id != 0) {
+		g_signal_handler_disconnect (gtk_window_get_screen (GTK_WINDOW (window)),
+					     details->size_changed_id);
+		details->size_changed_id = 0;
+	}
+
 	GTK_WIDGET_CLASS (nautilus_desktop_window_parent_class)->unrealize (widget);
 }
 
@@ -212,7 +214,7 @@ set_desktop_window_id (NautilusDesktopWindow *window,
 	root_window = gdk_screen_get_root_window (
 				gtk_window_get_screen (GTK_WINDOW (window)));
 
-	window_xid = GDK_WINDOW_XWINDOW (gdkwindow);
+	window_xid = GDK_WINDOW_XID (gdkwindow);
 
 	gdk_property_change (root_window,
 			     gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
@@ -224,8 +226,10 @@ static void
 realize (GtkWidget *widget)
 {
 	NautilusDesktopWindow *window;
+	NautilusDesktopWindowDetails *details;
 
 	window = NAUTILUS_DESKTOP_WINDOW (widget);
+	details = window->details;
 
 	/* Make sure we get keyboard events */
 	gtk_widget_set_events (widget, gtk_widget_get_events (widget) 
@@ -239,8 +243,9 @@ realize (GtkWidget *widget)
 
 	set_desktop_window_id (window, gtk_widget_get_window (widget));
 
-	g_signal_connect (gtk_window_get_screen (GTK_WINDOW (window)), "size_changed",
-			  G_CALLBACK (nautilus_desktop_window_screen_size_changed), window);
+	details->size_changed_id =
+		g_signal_connect (gtk_window_get_screen (GTK_WINDOW (window)), "size_changed",
+				  G_CALLBACK (nautilus_desktop_window_screen_size_changed), window);
 }
 
 static char *
@@ -257,20 +262,24 @@ real_get_icon (NautilusWindow *window,
 }
 
 static void
-nautilus_desktop_window_class_init (NautilusDesktopWindowClass *class)
+nautilus_desktop_window_class_init (NautilusDesktopWindowClass *klass)
 {
-	G_OBJECT_CLASS (class)->finalize = finalize;
-	GTK_WIDGET_CLASS (class)->realize = realize;
-	GTK_WIDGET_CLASS (class)->unrealize = unrealize;
+	GtkWidgetClass *wclass = GTK_WIDGET_CLASS (klass);
+	NautilusWindowClass *nclass = NAUTILUS_WINDOW_CLASS (klass);
 
+	wclass->realize = realize;
+	wclass->unrealize = unrealize;
+	wclass->map = map;
 
-	GTK_WIDGET_CLASS (class)->map = map;
-
-	NAUTILUS_WINDOW_CLASS (class)->window_type = NAUTILUS_WINDOW_DESKTOP;
-
-	NAUTILUS_WINDOW_CLASS (class)->get_title 
-		= real_get_title;
-	NAUTILUS_WINDOW_CLASS (class)->get_icon
-		= real_get_icon;
+	nclass->window_type = NAUTILUS_WINDOW_DESKTOP;
+	nclass->get_title = real_get_title;
+	nclass->get_icon = real_get_icon;
 
+	g_type_class_add_private (klass, sizeof (NautilusDesktopWindowDetails));
+}
+
+gboolean
+nautilus_desktop_window_loaded (NautilusDesktopWindow *window)
+{
+	return window->details->loaded;
 }
